unit FArray;



interface

uses Global;

  function ReadPrior0Set(var Value: TGenSet): Boolean;
  procedure WriteNext0Set(Value: TGenSet);
  procedure WriteAlt0Set(Value: TGenSet);
  procedure SwapArrays(PrevNext: Boolean);
  procedure ClearArrays;

implementation

uses Sysutils,
     Windows, SysConst;


const
  file1 = 'workfile1.tmp';
  file2 = 'workfile2.tmp';
  file3 = 'workfile3.tmp';
  SetSize = SizeOf(TGenSet);
//  DiskBufferSetCount = 1 shl 9;
//  MemoryBufferSetCount = 1 shl 20;
//  DiskBufferSize = DiskBufferSetCount * SetSize;
//  MemoryBufferSize = MemoryBufferSetCount * SetSize;

type
  TMixedArray = class
    FileName: String;
    DF: Integer;
    WriteState: Boolean;
    BIndex: Integer;
    CurrentLimit: Integer;
    Buffer: array of TGenSet;
    constructor Create(FName: String); virtual;
    destructor Destroy; override;
    function ReadSet(var Value: TGenSet): Boolean;
    procedure WriteSet(Value: TGenSet);
    procedure CloseArray;
  private
    function ReadFromFile: Boolean;
    function WriteToFile: Boolean;
    procedure ClearArray;
  end;
var
  PrevArray, NextArray, AltArray: TMixedArray;
  Active: Boolean;
  MemoryBufferSetCount: Integer;
  MemoryBufferSize: Integer;
  DiskBufferSize: Integer;

procedure SwapArrays(PrevNext: Boolean);
var s: TMixedArray;
begin
  if PrevNext then
  begin
    s := PrevArray;
    PrevArray := NextArray;
  end
  else
  begin
    s := AltArray;
    AltArray := NextArray;
  end;
  NextArray := s;
end;

function ReadPrior0Set(var Value: TGenSet): Boolean;
begin
  Result := PrevArray.ReadSet(Value);
  if not Result then
  begin
    PrevArray.CloseArray;
    NextArray.CloseArray;
    AltArray.CloseArray;
  end;
end;

procedure WriteNext0Set(Value: TGenSet);
begin
  NextArray.WriteSet(Value);
end;

procedure WriteAlt0Set(Value: TGenSet);
begin
  AltArray.WriteSet(Value);
end;

procedure ClearArrays;
begin
  if not Active then
  begin
    if MemoryBuffer > DiskBuffer
    then MemoryBufferSetCount := MemoryBuffer div DiskBuffer * DiskBuffer
    else MemoryBufferSetCount := DiskBuffer;
{    if MemoryBuffer > DiskBufferSetCount
    then MemoryBufferSetCount := MemoryBuffer div DiskBufferSetCount * DiskBufferSetCount
    else MemoryBufferSetCount := DiskBufferSetCount;             }
    DiskBufferSize := DiskBuffer * SetSize;
    MemoryBufferSize := MemoryBufferSetCount * SetSize;
    NextArray := TMixedArray.Create(file1);
    PrevArray := TMixedArray.Create(file2);
    AltArray := TMixedArray.Create(file3);
    Active := True;
  end;
  NextArray.ClearArray;
  NextArray.WriteSet([]);
  NextArray.CloseArray;
  PrevArray.ClearArray;
  AltArray.ClearArray;
end;

{ TMixedArray }

procedure TMixedArray.CloseArray;
begin
  if DF >=0 then
  begin
    if WriteState then WriteToFile;
    FileSeek(DF, 0, 0);
    CurrentLimit := 0;
  end
  else if WriteState then CurrentLimit := BIndex;
  WriteState := False;
  BIndex := 0;
end;

procedure TMixedArray.ClearArray;
begin
  if DF >=0 then FileClose(DF);
  if FileExists(FileName) then SysUtils.DeleteFile(FileName);
  DF := -1;
  WriteState := False;
  CurrentLimit := 0;
  BIndex := 0;
end;

constructor TMixedArray.Create(FName: String);
begin
  SetLength(Buffer, MemoryBufferSetCount);
  BIndex := 0;
  CurrentLimit := 0;
  FileName := FName;
  DF := -1;
end;

destructor TMixedArray.Destroy;
begin
   SetLength(Buffer, 0);
end;

function TMixedArray.ReadSet(var Value: TGenSet): Boolean;
begin
  Result := False;
  if BIndex >= CurrentLimit then
    if DF >= 0 then ReadFromFile else exit;
  Result := CurrentLimit <> 0;
  if Result then
  begin
    Value := Buffer[BIndex];
    BIndex := BIndex + 1;
  end;
end;

function TMixedArray.ReadFromFile: Boolean;
var
  I, CC: Integer;
begin
  Result := False;
  CurrentLimit := 0;
  BIndex := 0;
  for I := 0 to MemoryBufferSize div DiskBufferSize - 1 do
  begin
    CC := FileRead(DF, Buffer[I*DiskBuffer], DiskBufferSize);
    if CC >= 0 then
    begin
      CurrentLimit := CurrentLimit + (CC div SetSize);
      if CC < DiskBufferSize then break;
    end
    else exit;
  end;
  Result := True;
end;

function TMixedArray.WriteToFile: Boolean;
var
  I, J: Integer;
begin
  if DF = -1 then
  begin
    DF := Integer(CreateFile(PChar(FileName), GENERIC_READ or GENERIC_WRITE,
    0, nil, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL + FILE_FLAG_SEQUENTIAL_SCAN, 0));
  end;
  Result := DF >= 0;
  J := 0;
  for I := 0 to MemoryBufferSize div DiskBufferSize - 1 do
  begin
    J := J + DiskBuffer;
    if BIndex > J then
      Result := Result and
        (FileWrite(DF, Buffer[I*DiskBuffer], DiskBufferSize) >= 0)
    else
    begin
      Result := Result and
        (FileWrite(DF, Buffer[I*DiskBuffer], (BIndex-I*DiskBuffer)*SetSize) >= 0);
      break;
    end;
    if not Result then exit;
  end;
  BIndex := 0;
end;

procedure TMixedArray.WriteSet(Value: TGenSet);
begin
  if not WriteState and (DF >= 0) then
  begin
    FileClose(DF);
    DF := -1;
  end;
  WriteState := True;
  if BIndex >= MemoryBufferSetCount then WriteToFile;
  Buffer[BIndex] := Value;
  BIndex := BIndex + 1;
end;

initialization

  Active := False;
//  NextArray := TMixedArray.Create(file1);
//  PrevArray := TMixedArray.Create(file2);
//  AltArray := TMixedArray.Create(file3);

finalization

  NextArray.Free;
  PrevArray.Free;
  AltArray.Free;

end.
